#ifndef __CLOSEST_8BIT
#define __CLOSEST_8BIT

#include <wchar.h>
#include <locale.h>
#include <stdlib.h>
#include <stdio.h>
#include "palette.h"
#include "closest_lowbit.h"
#include "colorspace_convert.h"
#include "color_compare.h"

static void chb_get_ansi256_gray(chb_rgb * color, chb_palette_code id)
{
	chb_rgb gray = chb_get_palette_8(id);
	chb_rgb_copy(color, &gray);
}

static void chb_convert_256cube_to_rgb(chb_palette_code const cube_code, chb_rgb * rgb)
{
	chb_rgb color = chb_get_palette_8(cube_code);
	chb_rgb_copy(rgb, &color);
}


static void chb_closest_ansi256_gray(chb_rgb const * color, chb_palette_code * output_color)
{
	unsigned int big_value = 0xffffffff;
	unsigned int distance = big_value;
	*output_color = 232;
	chb_rgb test_col;

	for(unsigned int code = 232; code<=255; code++)
	{
		chb_get_ansi256_gray(&test_col, code);
		unsigned int diff = chb_rgb_diff(color, &test_col);

		if(diff < distance)
		{
			distance = diff;
			*output_color = code;
		}
	}
}

static chb_palette_code chb_convert256_cube_only(chb_rgb const * c)
{
	unsigned char cube[3];
	chb_palette_code cube_code;

	cube[0] = chb_rgb_get_channel(c, 0)/7/7;
	cube[1] = chb_rgb_get_channel(c, 1)/7/7;
	cube[2] = chb_rgb_get_channel(c, 2)/7/7;
	cube_code = 16 + (cube[0]*36 + cube[1]*6 + cube[2]);
	return cube_code;
}

static chb_palette_code chb_convert_256_xyzdiff(chb_rgb const * c)
{
	unsigned int distance = 0xffffffff;
	chb_rgb cube_color;
	chb_rgb cube_rgb;
	chb_rgb ansi16_rgb;
	chb_rgb gray_rgb;
	chb_palette_code cube_code;
	chb_palette_code output_code;
	float orig_rgb[3], test_rgb[3];
	float bestE, dE;
	chb_palette_code ansi_16_code;
	chb_palette_code ansi_256_gray;
	char bestC = 0;

	chb_to_float_rgb(c, orig_rgb);
	//printf("%i %i %i ", c[0], c[1], c[2]);
	//printf("%.1e %.1e %.1e ", orig_rgb[0], orig_rgb[1], orig_rgb[2]);

	///chb_rgb_to_ansi_color_cube(c, cube_color);
	///cube_code = 16 + (cube_color[0]*36 + cube_color[1]*6 + cube_color[2]);
	cube_code = chb_convert256_cube_only(c);
	output_code = cube_code; //default case

	///chb_ansi_color_cube_to_rgb(cube_color, cube_rgb);
	chb_convert_256cube_to_rgb(cube_code, &cube_rgb);
	chb_to_float_rgb(&cube_rgb, test_rgb);
	bestE = chb_compare_rgb_e(test_rgb, orig_rgb);
	//printf("%.1e,", bestE);

	for(int vary_comp=0; vary_comp<3; vary_comp++) {
		for(int offset=-1; offset<2; offset++) {
			if(offset==0)
				continue;
			for(int comp=0; comp<3; comp++) {
				if(comp==vary_comp) //is the component selected for jitter
					chb_rgb_set_channel(&cube_color, comp, chb_rgb_get_channel(c, comp)/7/7 + offset);
				else
					chb_rgb_set_channel(&cube_color, comp, chb_rgb_get_channel(c, comp)/7/7);
			}
			cube_code = 16 + (chb_rgb_get_channel(&cube_color, 0)*36 + chb_rgb_get_channel(&cube_color, 1)*6 + chb_rgb_get_channel(&cube_color, 2));

			//chb_ansi_color_cube_to_rgb(cube_color, cube_rgb);
			chb_convert_256cube_to_rgb(cube_code, &cube_rgb);
			chb_to_float_rgb(&cube_rgb, test_rgb);
			dE = chb_compare_rgb_e(test_rgb, orig_rgb);
			if(dE < bestE)
			{
				bestE = dE;
				output_code = cube_code;
			}
		}
	}

	chb_closest_color_4(c, &ansi_16_code);
	ansi16_rgb = chb_get_palette_4(ansi_16_code);
	chb_to_float_rgb(&ansi16_rgb, test_rgb);
	dE = chb_compare_rgb_e(test_rgb, orig_rgb);
	//printf("%.1e,", dE);
	if(dE < bestE)
	{
		bestC = 1;
		bestE = dE;
		output_code = ansi_16_code;
	}

	chb_closest_ansi256_gray(c, &ansi_256_gray);
	chb_get_ansi256_gray(&gray_rgb, ansi_256_gray);
	chb_to_float_rgb(&gray_rgb, test_rgb);
	dE = chb_compare_rgb_e(test_rgb, orig_rgb);
	//printf("%.1e ", dE);
	if(dE < bestE)
	{
		bestC = 2;
		bestE = dE;
		output_code = ansi_256_gray;
	}

	//printf("%c", 'A'+bestC);

	return output_code;
}

static chb_palette_code chb_convert256_simple_diff_only(chb_rgb const * c)
{
	unsigned int distance = 0xffffffff;
	unsigned int diff;
	chb_rgb  cube;
	chb_rgb ansi16;
	chb_rgb gray;
	chb_palette_code cube_code;
	chb_palette_code output_code = 0;
	chb_palette_code ansi_16_code;
	chb_palette_code ansi_256_gray;

	cube_code = chb_convert256_cube_only(c);
	chb_convert_256cube_to_rgb(cube_code, &cube);
	diff = chb_rgb_diff(&cube, c);

	if(diff < distance)
	{
		distance = diff;
		output_code = cube_code;
	}

	//these seem to break conversion, need more work :-(
	chb_closest_color_4(c, &ansi_16_code);
	ansi16 = chb_get_palette_4(ansi_16_code);
	diff = chb_rgb_diff(&ansi16, c);

	if(diff < distance)
	{
		distance = diff;
		output_code = ansi_16_code;
	}

	chb_closest_ansi256_gray(c, &ansi_256_gray);
	chb_get_ansi256_gray(&gray, ansi_256_gray);
	diff = chb_rgb_diff(&gray, c);

	if(diff < distance)
	{
		distance = diff;
		output_code = ansi_256_gray	;
	}

	return output_code;
}

static chb_palette_code chb_rgb_to_ansi256(chb_rgb const * c)
{
	//return chb_convert_256_xyzdiff(c);
	//return chb_convert256_simple_diff_only(c);
	return chb_convert256_cube_only(c);
}

static void chb_closest_color_8(chb_rgb const * color, chb_palette_code * output_code)
{
	*output_code = chb_rgb_to_ansi256(color);
}

#endif
