#include <stdlib.h>
#include <stdio.h>
#include "closest_8bit.h"
#include "print_output.h"
#include "chb.h"
#include "convert_palette.h"
#include "output_pixel.h"
#include "detect_color.h"
#include "detect_char.h"

#ifdef WIN32
#include <windows.h>
#endif

typedef void (*chb_output_func) (chb_rgb const *, chb_rgb const *);

chb_output_func chb_get_output_func(unsigned int CHB_DEPTH)
{
	chb_output_func output_func;

	switch(CHB_DEPTH) {
		case CHB_DEPTH_ANSI_4BIT:
			output_func = chb_output_ansi4bit_pixels;
			break;
		case CHB_DEPTH_ANSI_8BIT:
			output_func = chb_output_ansi8bit_pixels;
			break;
		case CHB_DEPTH_ANSI_24BIT:
			output_func = chb_output_ansi24bit_pixels;
			break;
#ifdef WIN32
		case CHB_DEPTH_MS_CONSOLE:
			output_func = chb_output_ms_console_pixels;
			break;
#endif
		default:
			output_func = chb_output_ansi3bit_pixels;
			break;
	}

	return output_func;
}

void chb_print_image_opt(unsigned int CHB_DEPTH, unsigned int CHB_CHAR, unsigned int width, unsigned int height, unsigned char const * rgb_image)
{
	if(CHB_DEPTH == CHB_DEPTH_BEST)
		CHB_DEPTH = chb_get_best_depth();
	if(CHB_CHAR == CHB_CHAR_BEST)
		CHB_CHAR = chb_get_best_char();

	chb_output_func output_func = chb_get_output_func(CHB_DEPTH);

	#ifdef WIN32
	unsigned int cp;
	cp = GetConsoleCP();
	if(CHB_CHAR == CHB_CHAR_CP437)
	{
		SetConsoleCP(1252);
	}
	#endif

	for(unsigned int row=0; row<height; row+=2)
	{
		char onLastOddRow = ((height % 2) != 0) && row == height-1;

		for(unsigned int col=0; col<width; col++)
		{
			int top_offset = (row*width+col)*3;
			int bot_offset = ((row+1)*width+col)*3;
			chb_rgb const * top_color;
			chb_rgb const * bot_color;

			top_color = (chb_rgb const*)&rgb_image[top_offset]; //requires uint8 and rgb ordering
			bot_color = NULL;

			if(!onLastOddRow)
				bot_color = (chb_rgb const*)&rgb_image[bot_offset];  //requires uint8 and rgb ordering

			output_func(top_color, bot_color);

			if(CHB_CHAR == CHB_CHAR_UTF8)
				chb_print_UHB_unicode_utf8();
			else if(CHB_CHAR == CHB_CHAR_UTF16)
				chb_print_UHB_unicode_utf8();
			else if(CHB_CHAR == CHB_CHAR_CP437)
				chb_print_UHB_cp_437();
			else
				chb_print_UHB_unicode_utf8();
		}
		if(CHB_DEPTH == CHB_DEPTH_MS_CONSOLE){
			#ifdef WIN32
			chb_set_ms_reset_code();
			#endif
		}
		else
			chb_print_ansi_reset();

		printf("\n");
	}

	#ifdef WIN32
	if(CHB_CHAR == CHB_CHAR_CP437)
		SetConsoleCP(cp);
	#endif

}

void chb_print_frame_opt(chb_frame_state * state, unsigned int CHB_DEPTH, unsigned int CHB_CHAR, unsigned int CHB_PALETTE, unsigned int CHB_REPLACE, unsigned int width, unsigned int height, unsigned char const * rgb_image)
{
	chb_cursor_move_top_left();
	chb_print_image_opt(CHB_DEPTH, CHB_CHAR, width, height, rgb_image);
}

void chb_print_image(unsigned int width, unsigned int height, unsigned char const * rgb_image)
{
	unsigned int chb_depth;
	unsigned int chb_char;
	chb_depth = chb_get_best_depth();
	chb_char = chb_get_best_char();

	chb_print_image_opt(chb_depth, chb_char, width, height, rgb_image);
}

void chb_print_frame(chb_frame_state * state, unsigned int width, unsigned int height, unsigned char const * rgb_image)
{
	unsigned int chb_depth;
	unsigned int chb_char;
	unsigned int chb_replace;
	unsigned int chb_palette;
	chb_depth = chb_get_best_depth();
	chb_char = chb_get_best_char();
	chb_replace = CHB_REPLACE_BEST; //chb_get_best_replace();
	chb_palette = CHB_PALETTE_BEST; //chb_get_best_palette();

	chb_print_frame_opt(state, chb_depth, chb_char, chb_palette, chb_replace, width, height, rgb_image);
}

void chb_init_frame(unsigned int width, unsigned int height, chb_frame_state* state){
	//does nothing yet
}

void chb_free_frame(chb_frame_state * state){
	//does nothing yet
}

