#ifndef __CHB_PALETTE
#define __CHB_PALETTE

#include "rgb.h"

typedef unsigned char chb_palette_code; //a palette index for a single color

#define CHB_PALETTE4_OFFSET 8
#define CHB_PALETTE8_CUBE_OFFSET 16
#define CHB_PALETTE8_GRAY_OFFSET 232

static unsigned char chb_palette_3c[24] = {
  0,   0,   0,
205,   0,   0,
  0, 205,   0,
205, 205,   0,
  0,   0, 238,
205,   0, 205,
  0, 205, 205,
229, 229, 229,
};

static unsigned char chb_palette_4c[24] = {
127, 127, 127,
255,   0,   0,
  0, 255,   0,
255, 255,   0,
 92,  92, 255,
255,   0, 255,
  0, 255, 255,
255, 255, 255,
};

static chb_rgb const * const chb_palette_3 = (chb_rgb const * const) &chb_palette_3c;
static chb_rgb const * const chb_palette_4 = (chb_rgb const * const) &chb_palette_4c;

static chb_rgb chb_get_palette_8_gray(unsigned char const gray_code)
{
	chb_rgb c;

	char code = gray_code;
	char val = code * 10 + 8;

	chb_rgb_set_r(&c, val);
	chb_rgb_set_g(&c, val);
	chb_rgb_set_b(&c, val);

	return c;
}

static chb_rgb chb_get_palette_8_cube(unsigned char const cube_code)
{
	chb_rgb c;

	unsigned char code = cube_code;
	unsigned char r = code / 36;
	unsigned char g = (code - (r*36)) / 6;
	unsigned char b = (code - (r*36) - (g*6));

	chb_rgb_set_r(&c, r*51);
	chb_rgb_set_g(&c, g*51);
	chb_rgb_set_b(&c, b*51);

	return c;
}

static chb_rgb chb_get_palette_3(chb_palette_code id)
{
	chb_rgb c;
	c = chb_palette_3[id];
	return c;
}

static chb_rgb chb_get_palette_4(chb_palette_code id)
{
	chb_rgb c;

	if(id < CHB_PALETTE4_OFFSET)
		c = chb_palette_3[id];
	else
		c = chb_palette_4[id-CHB_PALETTE4_OFFSET];

	return c;
}

/*
The 8 bit palette colors are:
0-15    : the 4bit palette colors
16-231  : a 6x6x6 rgb color cube with axes rxgxb (includes white + black)
232-255 : grayscale from almost black to almost white
This does not seem to be defined in ITU docs, but all terminals seem to do it.
*/
static chb_rgb chb_get_palette_8(chb_palette_code id)
{
	chb_rgb c;

	if(id < 8)
		c = chb_palette_3[id];
	else if(id < CHB_PALETTE8_CUBE_OFFSET)
		c = chb_palette_4[id];
	else if(id < CHB_PALETTE8_GRAY_OFFSET)
		c = chb_get_palette_8_cube(id-CHB_PALETTE8_CUBE_OFFSET);
	else
		c = chb_get_palette_8_gray(id-CHB_PALETTE8_GRAY_OFFSET);

	return c;
}

#endif
