#ifndef __CHB_RGB
#define __CHB_RGB

#include <stdlib.h>
#include "math_helper.h"

typedef struct chb_rgb
{
	unsigned char c[3];
} chb_rgb;

static void chb_rgb_set_channel(chb_rgb * rgb, unsigned char channel, unsigned char v)
{ rgb->c[channel] = v; }
static void chb_rgb_set_r(chb_rgb * rgb, unsigned char v)
{ chb_rgb_set_channel(rgb, 0, v); }
static void chb_rgb_set_g(chb_rgb * rgb, unsigned char v)
{ chb_rgb_set_channel(rgb, 1, v); }
static void chb_rgb_set_b(chb_rgb * rgb, unsigned char v)
{ chb_rgb_set_channel(rgb, 2, v); }

static void chb_rgb_set_rgb(chb_rgb * rgb, unsigned char const r, unsigned char const g, unsigned char const b)
{
	chb_rgb_set_r(rgb, r);
	chb_rgb_set_g(rgb, g);
	chb_rgb_set_b(rgb, b);
}

static void chb_rgb_set_rgbv(chb_rgb * rgb, unsigned char const * const v)
{
	chb_rgb_set_r(rgb, v[0]);
	chb_rgb_set_g(rgb, v[1]);
	chb_rgb_set_b(rgb, v[2]);
}

static unsigned char chb_rgb_get_channel(chb_rgb const * rgb, unsigned char channel)
{ return rgb->c[channel]; }
static unsigned char chb_rgb_get_r(chb_rgb const * const rgb)
{ return chb_rgb_get_channel(rgb, 0); }
static unsigned char chb_rgb_get_g(chb_rgb const * const rgb)
{ return chb_rgb_get_channel(rgb, 1); }
static unsigned char chb_rgb_get_b(chb_rgb const * const rgb)
{ return chb_rgb_get_channel(rgb, 2); }

static void chb_rgb_copy(chb_rgb * dest, chb_rgb const * src)
{
	for(unsigned char c=0; c<3; c++)
		chb_rgb_set_channel(dest, c, chb_rgb_get_channel(src, c));
}

static unsigned int chb_rgb_diff(chb_rgb const * const a, chb_rgb const * const b)
{
	unsigned int diff = 0;
	for(unsigned int i=0; i<3; i++)
	{
		unsigned char ac = chb_rgb_get_channel(a, i);
		unsigned char bc = chb_rgb_get_channel(b, i);
		//diff += abs(ac-bc);
		diff = CHB_MAX(diff, (unsigned char)abs(ac-bc));
	}

	return diff;
}

#endif
