#include <wchar.h>
#include <locale.h>
#include <stdlib.h>
#include <stdio.h>
#include "chb.h"

#define SIZE 32

static void chb_set_uint8_color(unsigned char * c, unsigned char r, unsigned char g, unsigned char b)
{
	c[0] = r;
	c[1] = g;
	c[2] = b;
}

void chb_print_test_image_small()
{
	unsigned char * image = (unsigned char *)malloc(sizeof(unsigned char) * 4 * 2 * 3);
	if(image == NULL) {
		printf("Could not allocate for image\n");
		return;
	}

	chb_set_uint8_color(&image[0], 255,   0,   0);
	chb_set_uint8_color(&image[3],   0, 255,   0);
	chb_set_uint8_color(&image[6],   0,   0, 255);
	chb_set_uint8_color(&image[9], 255, 255, 255);
	chb_set_uint8_color(&image[12],   0, 255, 255);
	chb_set_uint8_color(&image[15], 255,   0, 255);
	chb_set_uint8_color(&image[18], 255, 255,   0);
	chb_set_uint8_color(&image[21],   0,   0,   0);

	printf("small test image (rgbw, cmyk)\n");
	chb_print_image_opt(CHB_DEPTH_ANSI_3BIT, CHB_CHAR_BEST, 4, 2, image);
	chb_print_image_opt(CHB_DEPTH_ANSI_4BIT, CHB_CHAR_BEST, 4, 2, image);
	chb_print_image_opt(CHB_DEPTH_ANSI_8BIT, CHB_CHAR_BEST, 4, 2, image);
	chb_print_image_opt(CHB_DEPTH_ANSI_24BIT, CHB_CHAR_BEST, 4, 2, image);
	free(image);
}

void chb_black_magenta_white_green(unsigned char* c, int x, int y)
{
	c[0] = y*(255/SIZE);
	c[1] = x*(255/SIZE);
	c[2] = 255 - (SIZE-y)*(255/SIZE)/2 - (SIZE-x)*(255/SIZE)/2;
}

void chb_blue_red_yellow_green(unsigned char* c, int x, int y)
{
	int b;
	c[0] = y*(255/SIZE);
	c[1] = x*(255/SIZE);
	b = 255 - (y)*(255/SIZE) - (x)*(255/SIZE);
	c[2] = 0; if(b > 0) c[2] = b;
}

void chb_blue_red_black_green(unsigned char* c, int x, int y)
{
	int r, g, b;
	r = 255 - (SIZE-y)*(255/SIZE) - (x)*(255/SIZE);
	g = 255 - (y)*(255/SIZE) - (SIZE-x)*(255/SIZE);
	b = 255 - (y)*(255/SIZE) - (x)*(255/SIZE);
	c[0] = 0; if(r > 0) c[0] = r;
	c[1] = 0; if(g > 0) c[1] = g;
	c[2] = 0; if(b > 0) c[2] = b;
}
void chb_blue_magenta_yellow_green(unsigned char* c, int x, int y)
{
	c[0] = y*(255/SIZE);
	c[1] = x*(255/SIZE);
	c[2] = (SIZE-y)*(255/SIZE)/2 + (SIZE-x)*(255/SIZE)/2;
}

void chb_print_test_image_large()
{
	unsigned char * image = (unsigned char *)malloc(sizeof(unsigned char) * SIZE * SIZE * 3);
	if(image == NULL) {
		printf("Could not allocate for image\n");
		return;
	}

	for(int i=0; i<SIZE; i++)
	{
		for(int j=0; j<SIZE; j++)
		{
			unsigned char * c = &image[(i*SIZE+j)*3];
			chb_black_magenta_white_green(c, j, i);
			chb_blue_red_yellow_green(c, j, i);
			chb_blue_red_black_green(c, j, i);
			chb_blue_magenta_yellow_green(c, j, i);
		}
	}

	printf("large test image\n");
	chb_print_image_opt(CHB_DEPTH_ANSI_3BIT, CHB_CHAR_BEST, SIZE, SIZE, image);
	chb_print_image_opt(CHB_DEPTH_ANSI_4BIT, CHB_CHAR_BEST, SIZE, SIZE, image);
	chb_print_image_opt(CHB_DEPTH_ANSI_8BIT, CHB_CHAR_BEST, SIZE, SIZE, image);
	chb_print_image_opt(CHB_DEPTH_ANSI_24BIT, CHB_CHAR_BEST, SIZE, SIZE, image);
	free(image);
}

int main(int i, char** c)
{
	chb_print_test_image_small();
	chb_print_test_image_large();

	return 0;
}


