#include <stdlib.h>

#ifdef __linux
#include <wand/MagickWand.h>
#endif

#include "view_impl.h"


#define ThrowWandException(wand) \
{ \
	char \
		*description; \
	\
	ExceptionType \
		severity; \
	\
	description=MagickGetException(wand,&severity); \
	(void) fprintf(stderr,"%s %s %lu %s\n",GetMagickModule(),description); \
	description=(char *) MagickRelinquishMemory(description); \
	exit(-1); \
}

void getWandImage(char const * filepath, size_t * out_width, size_t * out_height, unsigned char * * rgb_image)
{
	MagickBooleanType status;
	MagickPixelPacket pixel;
	MagickWand *image_wand;
	PixelIterator *iterator;
	PixelWand **pixels;
	register ssize_t x;
	size_t width;
	ssize_t y;
	double r, g, b;

	MagickWandGenesis();
	image_wand=NewMagickWand();
	status=MagickReadImage(image_wand, filepath);

	if (status == MagickFalse)
	{
		printf("Error opening %s\n", filepath);
		exit(1);

		ThrowWandException(image_wand);
	}

	iterator=NewPixelIterator(image_wand);

	if (iterator == (PixelIterator *) NULL)
	{
		printf("Error opening %s\n", filepath);
		exit(1);

		ThrowWandException(image_wand);
	}

	size_t height = MagickGetImageHeight(image_wand);
	width = MagickGetImageWidth(image_wand);
	*out_width = width;
	*out_height = height;
	(*rgb_image) = malloc(height * width * 3 * sizeof(unsigned char));

	for (y=0; y < (ssize_t) height; y++)
	{
		pixels=PixelGetNextIteratorRow(iterator,&width);
		if (pixels == (PixelWand **) NULL)
			break;

		for (x=0; x < (ssize_t) width; x++)
		{
			size_t offset = (y*(width) + x) * 3;
			PixelGetMagickColor(pixels[x],&pixel);
			r = pixel.red   / 255.0;
			g = pixel.green / 255.0;
			b = pixel.blue  / 255.0;
			if(r > 255.0) r = 255.0;
			if(g > 255.0) g = 255.0;
			if(b > 255.0) b = 255.0;
//printf("%f %f %f\n", r, g, b);
			(*rgb_image)[ offset+0 ] = r;
			(*rgb_image)[ offset+1 ] = g;
			(*rgb_image)[ offset+2 ] = b;
		}
	}

	if (y < (ssize_t) MagickGetImageHeight(image_wand))
	{
		printf("Error opening %s\n", filepath);
		exit(1);

		ThrowWandException(image_wand);
	}
	iterator=DestroyPixelIterator(iterator);
	image_wand=DestroyMagickWand(image_wand);
}

int main(int argc, char ** argv)
{
	showImages(argc, argv, getWandImage);
	return 0;
}

