
#ifdef __APPLE__
#import <CoreFoundation/CoreFoundation.h>
#import <Foundation/Foundation.h>
#endif

#include "view_impl.h"

void getCGImage(char const * filepath, size_t * out_width, size_t * out_height, unsigned char * * rgb_image)
{
	NSString *path = [NSString stringWithCString:filepath encoding: NSUTF8StringEncoding];
	NSURL *url = [NSURL fileURLWithPath:path];
	CGImageSourceRef src = CGImageSourceCreateWithURL( (CFURLRef)url, NULL);

	if(src == NULL)
	{
		printf("Error opening %s\n", filepath);
		exit(1);
	}

	CGImageRef imageRef = CGImageSourceCreateImageAtIndex(src, 0, nil);
	size_t width = CGImageGetWidth(imageRef);
	size_t height = CGImageGetHeight(imageRef);
	(*out_width) = width;
	(*out_height) = height;
	

	//from http://stackoverflow.com/questions/448125/how-to-get-pixel-data-from-a-uiimage-cocoa-touch-or-cgimage-core-graphics#1262893
	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	unsigned char *rawData = (unsigned char*) calloc(height * width * 4, sizeof(unsigned char));

	NSUInteger bytesPerPixel = 4;
	NSUInteger bytesPerRow = bytesPerPixel * width;
	NSUInteger bitsPerComponent = 8;
	CGContextRef context = CGBitmapContextCreate(rawData, width, height,
			bitsPerComponent, bytesPerRow, colorSpace,
			kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
	CGColorSpaceRelease(colorSpace);

	CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
	CGContextRelease(context);

/*
	NSUInteger byteIndex = (bytesPerRow * y) + x * bytesPerPixel;
	for (int i = 0 ; i < count ; ++i)
	{
		CGFloat red   = (rawData[byteIndex]     * 1.0) / 255.0;
		CGFloat green = (rawData[byteIndex + 1] * 1.0) / 255.0;
		CGFloat blue  = (rawData[byteIndex + 2] * 1.0) / 255.0;
		CGFloat alpha = (rawData[byteIndex + 3] * 1.0) / 255.0;
		byteIndex += bytesPerPixel;

		UIColor *acolor = [UIColor colorWithRed:red green:green blue:blue alpha:alpha];
		[result addObject:acolor];
	}
	free(rawData);
*/

	(*rgb_image) = malloc(height * width * 3 * sizeof(char));
	NSUInteger byteIndex = 0;//(bytesPerRow * y) + x * bytesPerPixel;
	int count = width * height;
	for (int i = 0 ; i < count ; ++i)
	{
		(*rgb_image)[i*3 + 0] = (rawData[byteIndex]     * 1.0);
		(*rgb_image)[i*3 + 1] = (rawData[byteIndex + 1] * 1.0);
		(*rgb_image)[i*3 + 2] = (rawData[byteIndex + 2] * 1.0);
		byteIndex += bytesPerPixel;
	}
	CGImageRelease(imageRef);
}

int main(int argc, char ** argv)
{
	showImages(argc, argv, getCGImage);
	return 0;
}

