#ifndef __VIEW_RESIZE_H
#define __VIEW_RESIZE_H

static float scale_to_canvas(unsigned int src_w, unsigned int src_h, unsigned int canvas_w, unsigned int canvas_h)
{
	float scale = 1.0f;
	float src_ar = src_w / (float) src_h;
	float canvas_ar = canvas_w / (float) canvas_h;
	char src_wider = src_ar > canvas_ar;

	if(src_wider)
		scale = canvas_w / (float) src_w;
	else
		scale = canvas_h / (float) src_h;

	return scale;
}

static unsigned int get_target_w(unsigned int src_w, unsigned int src_h, unsigned int canvas_w, unsigned int canvas_h)
{
	float scale = scale_to_canvas(src_w, src_h, canvas_w, canvas_h);
	if(scale > 1.0f)
		return src_w;
	unsigned int target = (unsigned int) (src_w * scale);
	return target;
}

static unsigned int get_target_h(unsigned int src_w, unsigned int src_h, unsigned int canvas_w, unsigned int canvas_h)
{
	float scale = scale_to_canvas(src_w, src_h, canvas_w, canvas_h);
	if(scale > 1.0f)
		return src_h;
	unsigned int target = (unsigned int) (src_h * scale);
	return target;
}

static unsigned char * resize_rgb_image(unsigned int in_w, unsigned int in_h, unsigned char const * in_rgb, unsigned int out_w, unsigned int out_h)
{
	//assert(in_w > 0 && in_h > 0 && out_w > 0 && out_h > 0);

	float ratio_w = (in_w / (float)out_w);
	float ratio_h = (in_h / (float)out_h);
	unsigned char * out_rgb = (unsigned char*) malloc( out_w * out_h * 3 * sizeof(unsigned char));
	if(out_rgb == NULL) {
		printf("Error resizing image\n");
		exit(1);
	}

	for(unsigned int j=0; j<out_h; j++)
	{
		unsigned int src_h1 = (unsigned int) (ratio_h *  j);
		unsigned int src_h2 = (unsigned int) (ratio_h * (j+1));
		//unsigned int src_h1 = in_h *  j    / out_h;
		//unsigned int src_h2 = in_h * (j+1) / out_h;
		unsigned int kernel_h = src_h2-src_h1;

		for(unsigned int i=0; i<out_w; i++)
		{
			unsigned int src_w1 = (unsigned int) (ratio_w *  i);
			unsigned int src_w2 = (unsigned int) (ratio_w * (i+1));
			//unsigned int src_w1 = in_w *  i    / out_w;
			//unsigned int src_w2 = in_w * (i+1) / out_w;
			unsigned int kernel_w = src_w2-src_w1;

			unsigned int dst_pixel = (j*out_w+i)*3;
			unsigned int color[3];
			float box_scale = 1.0f / (kernel_w*kernel_h);
			//printf("%ux%u (%u.%u)x(%u.%u) : %f (%ux%u)\n", i, j, src_w1, src_w2, src_h1, src_h2, box_scale, kernel_w, kernel_h);

			for(int p=0; p<3; p++)
			{
				color[p] = 0;
			}

			for(unsigned int r=src_h1; r<src_h2; r++)
			{
				for(unsigned int c=src_w1; c<src_w2; c++)
				{
					unsigned int src_pixel = (r*in_w+c)*3;
					for(unsigned int p=0; p<3; p++)
					{
						color[p] += in_rgb[src_pixel+p];
					}
				}
			}

			for(int p=0; p<3; p++)
			{
				out_rgb[dst_pixel+p] = (unsigned char) (color[p] * box_scale);
			}
		}
	}

	return out_rgb;
}

#endif

